<?php
namespace PAGEmachine\Ats\Tests\Unit\Controller\Application;

/*
 * This file is part of the PAGEmachine ATS project.
 */

use Nimut\TestingFramework\TestCase\UnitTestCase;
use PAGEmachine\Ats\Controller\Application\SubmitController;
use PAGEmachine\Ats\Domain\Model\Application;
use PAGEmachine\Ats\Domain\Repository\ApplicationRepository;
use PAGEmachine\Ats\Message\AcknowledgeMessage;
use PAGEmachine\Ats\Message\MessageFactory;
use PAGEmachine\Ats\Service\ExtconfService;
use Prophecy\Argument;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\RequestInterface;
use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;

/**
 * Testcase for SubmitController
 */
class SubmitControllerTest extends UnitTestCase
{
    /**
     * @var SubmitController
    */
    protected $controller;

    /**
     * @var ViewInterface|Prophecy\Prophecy\ObjectProphecy
     */
    protected $view;


    /**
     * @var Application|Prophecy\Prophecy\ObjectProphecy
     */
    protected $application;

    /**
     * @var MessageFactory
     */
    protected $messageFactory;

    /**
     * Set up this testcase
    */
    public function setUp()
    {

        $this->controller = $this->getMockBuilder(SubmitController::class)->setMethods([
            'redirect',
            'forward',
            ])->getMock();

        $this->application = $this->prophesize(Application::class);

        $this->view = $this->prophesize(ViewInterface::class);
        $this->inject($this->controller, 'view', $this->view->reveal());

        $this->messageFactory = $this->prophesize(MessageFactory::class);
        $this->inject($this->controller, "messageFactory", $this->messageFactory->reveal());
    }

    public function tearDown()
    {
        GeneralUtility::purgeInstances();
    }

    /**
     * @test
     */
    public function showsSummary()
    {

        $this->view->assign('application', $this->application->reveal())->shouldBeCalled();

        $this->controller->showSummaryAction($this->application->reveal());
    }

    /**
     * @test
     */
    public function submitsApplication()
    {

        $this->application->setApplicationType(1)->shouldBeCalled();
        $this->application->submit()->shouldBeCalled();

        $repository = $this->prophesize(ApplicationRepository::class);
        $this->inject($this->controller, "repository", $repository->reveal());

        $request = $this->prophesize(RequestInterface::class);
        $request->getArgument('application')->willReturn([]);
        $this->inject($this->controller, 'request', $request->reveal());

        $repository->updateAndLog(
            $this->application->reveal(),
            'new'
        )->shouldBeCalled();

        $extconfService = $this->prophesize(ExtconfService::class);
        $extconfService->getSendAutoAcknowledge()->willReturn(true);

        GeneralUtility::setSingletonInstance(ExtconfService::class, $extconfService->reveal());

        $message = $this->prophesize(AcknowledgeMessage::class);
        $message->setUseBackendUserCredentials(false)->shouldBeCalled();
        $message->applyAutoAcknowledgeTemplate()->shouldBeCalled()->willReturn(true);
        $message->getApplication()->willReturn($this->application);
        $message->getRenderedSubject()->willReturn("foo");
        $message->getCc()->willReturn("cc");
        $message->getBcc()->willReturn("bcc");
        $message->getRenderedBody()->willReturn("Body");
        $message->getSendType()->willReturn("mail");
        $message->send()->shouldBeCalled();

        $repository->updateAndLog(
            $this->application->reveal(),
            'autoAcknowledge',
            Argument::type("array")
        )->shouldBeCalled();

        $this->messageFactory->createMessage("acknowledge", $this->application)->shouldBeCalled()->willReturn($message->reveal());

        $this->controller->submitAction($this->application->reveal());
    }
}
